==> locus-0.88.tar.gz: locus-0.88/lib/fbase/dynstem.cpp <==
#include "myfndt.h"
#include "dynstem.h"

#define PROTO(rtype, name, par)		typedef rtype (*F##name)par;
#include "libstem.h"

#include "locusopt.h"
#include "ostem.h"
#include "odlerr.h"
#include <dlfcn.h>

CDynamicStemmer::CDynamicStemmer(CLocusOptions &Options):
    Lib(NULL),
    Stemmer(NULL),
    ApplyProc(NULL),
    DoneProc(NULL),
    DlsymErrMsgFormat(Options.Get(T_DLSYMERR_SS))
{    
    try
    {   InitLib(Options.Get(T_DLOPENERR_SS));
        FGeneric InitProc = GetFncAddr("StemInit");
        Stemmer = (reinterpret_cast<FStemInit>(InitProc))(Options.Get(O_SDKEY),
            Options.Get(O_SDSUPP),
  	        Options.Get(O_SDDATA),
	        Options.Get(O_SDTABLE));
        ApplyProc = GetFncAddr("StemApply");
        DoneProc = GetFncAddr("StemDone");
    }
    catch (...)
    {   Done();
        throw;
    }
}

CDynamicStemmer::~CDynamicStemmer()
{
    Done();
}

void CDynamicStemmer::Done()
{
    if (Lib)
    {   if (DoneProc && Stemmer)
            (reinterpret_cast<FStemDone>(DoneProc))(Stemmer);
        dlclose(Lib);
    }
}

int CDynamicStemmer::Apply(DString &WordForm)
{
    return (reinterpret_cast<FStemApply>(ApplyProc))(Stemmer, WordForm);
}

void CDynamicStemmer::InitLib(const DString &ErrMsgFormat)
{   static const char *LibNameHead = "libstem.so.";

    CMemPtr<char> LibName(strlen(LibNameHead) + LIBSTEM_MAJOR_VERSION + 1);
    sprintf((char *)LibName, "%s%d", LibNameHead, LIBSTEM_MAJOR_VERSION);

    Lib = dlopen((char *)LibName, RTLD_LAZY | RTLD_GLOBAL);
    if (!Lib)
    {   DString err = dlerror();
        CMemPtr<char> Msg(ErrMsgFormat.GetLength() +
            strlen((char *)LibName) +
            err.GetLength() +
            10);
        sprintf((char *)Msg, ErrMsgFormat.c_str(), (char *)LibName, err.c_str());
        throw CGeneralException((char *)Msg);
    }
}

CDynamicStemmer::FGeneric CDynamicStemmer::GetFncAddr(const char *FncName)
{
    void *Addr = dlsym(Lib, FncName);
    if (!Addr)
    {   DString err = dlerror();
        CMemPtr<char> Msg(DlsymErrMsgFormat.GetLength() +
            strlen(FncName) +
            err.GetLength() +
            10);
        sprintf((char *)Msg, DlsymErrMsgFormat.c_str(), FncName, err.c_str());
        throw CGeneralException((char *)Msg);
    }
    return reinterpret_cast<FGeneric>(Addr);
}

==> locus-0.92.tar.gz: locus-0.92/lib/fbase/dynstem.cpp <==
#include "myfndt.h"
#include "dynstem.h"
#include "stemimpl.h"

#define PROTO(rtype, name, par)		typedef rtype (*F##name)par;
#include "libstem.h"

#include "locusopt.h"
#include "odlerr.h"
#include "ouno.h"
#include "ometastm.h"
#include <dlfcn.h>

CDynamicStemmer::CDynamicStemmer(CLocusOptions &Options):
    Lib(NULL),
    Stemmer(NULL),
    ApplyProc(NULL),
    DoneProc(NULL),
    DlsymErrMsgFormat(Options.Get(T_DLSYMERR_SS))
{    
    try
    {   InitLib(GetLibraryName(Options.Get(O_STEMLIB)),
	    Options.Get(T_DLOPENERR_SS));
        FGeneric InitProc = GetFncAddr("StemInit");
        Stemmer = (reinterpret_cast<FStemInit>(InitProc))(
	    Options.Get(O_UNOFILE));
        ApplyProc = GetFncAddr("StemApply");
        DoneProc = GetFncAddr("StemDone");
    }
    catch (...)
    {   Done();
        throw;
    }
}

DString CDynamicStemmer::GetLibraryName(const DString &StemImpl)
{
    for (int i = 0;
	i < sizeof(StemImplementations) / sizeof(StemImplementations[0]);
	++i)
    {   if (StemImpl == StemImplementations[i])
        {   CMemPtr<char> LibName(StemImpl.GetLength() + 20);
	    sprintf((char *)LibName, "%s.so.%d",
		StemImpl.c_str(),
		StemLibraries[i]);
	    return DString((char *)LibName);
	}
		
    }
    Assert(0);
    return DString("");
}

CDynamicStemmer::~CDynamicStemmer()
{
    Done();
}

void CDynamicStemmer::Done()
{
    if (Lib)
    {   if (DoneProc && Stemmer)
            (reinterpret_cast<FStemDone>(DoneProc))(Stemmer);
        dlclose(Lib);
    }
}

int CDynamicStemmer::Apply(DString &WordForm)
{
    return (reinterpret_cast<FStemApply>(ApplyProc))(Stemmer, WordForm);
}

void CDynamicStemmer::InitLib(const DString &LibName,
    const DString &ErrMsgFormat)
{   
    Lib = dlopen(LibName.c_str(), RTLD_LAZY | RTLD_GLOBAL);
    if (!Lib)
    {   DString err = dlerror();
        CMemPtr<char> Msg(ErrMsgFormat.GetLength() +
            LibName.GetLength() +
            err.GetLength() +
            10);
        sprintf((char *)Msg,
	    ErrMsgFormat.c_str(),
	    LibName.c_str(),
	    err.c_str());
        throw CGeneralException((char *)Msg);
    }
}

CDynamicStemmer::FGeneric CDynamicStemmer::GetFncAddr(const char *FncName)
{
    void *Addr = dlsym(Lib, FncName);
    if (!Addr)
    {   DString err = dlerror();
        CMemPtr<char> Msg(DlsymErrMsgFormat.GetLength() +
            strlen(FncName) +
            err.GetLength() +
            10);
        sprintf((char *)Msg, DlsymErrMsgFormat.c_str(), FncName, err.c_str());
        throw CGeneralException((char *)Msg);
    }
    return reinterpret_cast<FGeneric>(Addr);
}
